<?php

namespace Drupal\lightbox_tobii;

use Drupal\Core\Field\FormatterInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Yaml\Exception\ParseException;
use Symfony\Component\Yaml\Yaml;

/**
 * Utility class for Lightbox Tobii.
 */
class LightboxTobiiUtils {

  /**
   * Attaches the Tobii library and its settings to the render elements.
   *
   * Merges global and formatter-specific Tobii settings (in YAML format),
   * attaches them to the `drupalSettings` array, and adds the Tobii library.
   *
   * @param \Drupal\Core\Field\FormatterInterface $formatter
   *   The field formatter instance.
   * @param array $elements
   *   The render array elements to which the library and settings will be
   *   attached.
   */
  public static function attachLibraryAndSettings(FormatterInterface $formatter, array &$elements) {

    $config = \Drupal::config('lightbox_tobii.settings');
    $module_settings_yaml = $config->get('tobii_settings') ?: '';
    $module_settings = Yaml::parse($module_settings_yaml) ?? [];

    $formatter_settings = $formatter->getSettings();
    $formatter_settings_yaml = $formatter_settings['tobii_settings'] ?: '';
    $formatter_settings = Yaml::parse($formatter_settings_yaml) ?? [];

    $tobii_settings = array_merge($module_settings, $formatter_settings);

    // Define a default configuration with module settings.
    $elements['#attached']['drupalSettings']['lightbox_tobii']['default'] = $module_settings;
    if (count($tobii_settings) > 0) {
      $conf_id = $formatter->getConfigurationId();
      // Define a formatter specific configuration.
      $elements['#attached']['drupalSettings']['lightbox_tobii'][$conf_id] = $tobii_settings;
    }
    $elements['#attached']['library'][] = 'lightbox_tobii/tobii';
  }

  /**
   * Validate the Yaml settings.
   */
  public static function validateYamlSettings(array &$element, FormStateInterface $form_state, array &$complete_form) {
    // Use Symfony YAML component for validation.
    try {
      $element_path = $element['#parents'];
      $element_name = $element_path[0];
      for ($i = 1; $i < count($element_path); $i++) {
        $element_name .= '[' . $element_path[$i] . ']';
      }
      $yaml_input = $form_state->getValue($element_path);
      // No check if yaml string is empty.
      if (empty($yaml_input)) {
        return;
      }
      // If it's invalid, it will throw an exception.
      $parsed_yaml = Yaml::parse($yaml_input);
      // Check if the parsed YAML is null or empty.
      if (empty($parsed_yaml)) {
        $form_state->setErrorByName($element_name, t('The YAML input is empty.'));
      }
    }
    catch (ParseException $e) {
      // Catch the parse exception and set the form error.
      $form_state->setErrorByName($element_name, t('The YAML provided is invalid: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
  }

}
