<?php

namespace Drupal\lightbox_tobii\Plugin\Field\FieldFormatter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\lightbox_tobii\LightboxTobiiUtils;
use Symfony\Component\Yaml\Yaml;

/**
 * Trait for lightbox formatters.
 */
trait LightboxTobiiFormatterTrait {

  /**
   * Define the default settings for the formatter.
   */
  public static function defaultSettings() {
    return self::getTobiiSettings() + parent::defaultSettings();
  }

  /**
   * Returns the Tobii settings for the formatter.
   *
   * @return array
   *   An associative array containing the Tobii settings.
   */
  public static function getTobiiSettings() {
    return ['tobii_settings' => ''];
  }

  /**
   * Define the settings form.
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Initialize the form array with parent's settings form.
    $form = parent::settingsForm($form, $form_state);

    $this->tobiiSettingsForm($form);

    return $form;
  }

  /**
   * Adds the Tobii settings textarea to the formatter settings form.
   *
   * @param array $form
   *   The form array to be altered.
   */
  public function tobiiSettingsForm(array &$form) {
    $form['tobii_settings'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Tobii settings'),
      '#default_value' => $this->getSetting('tobii_settings'),
      '#description' => $this->t('Enter the Tobii settings in YAML format.'),
      '#rows' => 5,
      '#element_validate' => [[LightboxTobiiUtils::class, 'validateYamlSettings']],
    ];
  }

  /**
   * Define the summary of the settings.
   */
  public function settingsSummary() {
    // Retrieve the summary from the parent class.
    $summary = parent::settingsSummary();

    $this->tobiiSettingsSummary($summary);

    return $summary;
  }

  /**
   * Adds summary items for the formatter settings.
   *
   * @param array $summary
   *   The summary items array to be altered.
   */
  public function tobiiSettingsSummary(array &$summary) {
    $formatter_settings = $this->getSettings();
    $settings_yaml = $formatter_settings['tobii_settings'];

    if (!empty($settings_yaml)) {
      $tobii_settings = Yaml::parse($settings_yaml);
      if (isset($tobii_settings['captions'])) {
        $summary[] = $this->t('Tobii - Display captions: @captions', ['@captions' => $tobii_settings['captions'] ? $this->t('Yes') : $this->t('No')]);
      }
    }

    $summary[] = $this->t(
      '<em>Check the <a href=":url">Tobii settings</a> for the default configuration.</em>',
      [':url' => Url::fromRoute('lightbox_tobii.settings')->toString()]
    );
  }

  /**
   * Attaches the Tobii library and its settings to the render elements.
   *
   * Calls the utility to merge and attach global and formatter-specific
   * Tobii settings, and adds the Tobii library to the render array.
   *
   * @param array $elements
   *   The render array elements to which the library and settings will
   *   be attached.
   */
  protected function attachLibraryAndSettings(array &$elements) {
    LightboxTobiiUtils::attachLibraryAndSettings($this, $elements);
  }

}
